      subroutine bmain(header,round_aper,density,
     $     focallength,infilename,bhimaglim,blomaglim,
     $     rhimaglim,rlomaglim,ihimaglim,ilomaglim,outstyle,
     $     outfilename,batch,pixelsize,ascending,sortfield,
     $     catmap,error)
C
C $Id: bmain.f,v 1.4 1997/01/09 19:10:04 koehn Exp $
C     
C Description:
C     
C This program is the main routine controlling the batch mode
C of the asternet program.
C
C Programming interface:
C
C Name         In/out  Type Structure Meaning
C     
C header       in      logical        TRUE if headers are required in 
C                                     the ASCII output style
C round_aper   in      logical        TRUE if the search aperture is circular
C                                     FALSE if rectangular
C density      in      real*8         Maximum number of stars per square deg.
C focallength  in      real*8         Focal length of telescope
C infilename   in      character*64   Name of region specification file.
C bhimaglim    in      real*8         Dim blue star limit
C blomaglim    in      real*8         Bright blue star limit
C rhimaglim    in      real*8         Dim red star limit
C rlomaglim    in      real*8         Bright red star limit
C ihimaglim    in      real*8         Dim index star limit
C ilomaglim    in      real*8         Bright index star limit
C outstyle     in      character*1    Format for results
C                                     'a' is ASCII
C                                     'i' is IRAF
C                                     'c' is catalog
C outfilename  in      character*64   Prototype name of results file
C batch        in      integer        1 if batch mode
C                                     0 or -1 of interactive mode
C                                     Not used in this routine but
C                                     included for compatibility purposes.
C pixelsize    in      real*8         Pixel size of CCD. (Same units as
C                                     'focallength'
C ascending    in      logcial        TRUE if sort order is ascending
C sortfield    in      character*1    Selects which result parameter is to
C                                     be sorted
C                                     'R' is RA
C                                     'D' is Decl.
C                                     'd' is distance from aperture center
C                                     'b' is blue magnitude
C                                     'r' is red magnitude
C                                     'a' is aperture angle from North 
C                                     'i' is color index (blue - red) 
C capmap       in      character*64   Name of the catalog map file.
C error        out     logical        TRUE if an error occurs
C
C Errors:
C
C This routine acts as the interface to the region specification file.
C As such, it detects errors in the user supplied region specification
C list. There can be a variety of errors. This routine attempts to 
C detect them all but the variety of user supplied errors is limitless
C so, no doubt, some have been missed. In any case, when an error is
C detected, an "informative" error message is supplied, the routine
C sets 'error' TRUE, and then the routine immediately terminates.
C Also, if the subroutines that find the requested stars or the output
C subroutines return an error, this routine terminates immediately.
C
C Routines accessed
C
C get_next_token
C err_msg
C find_stars
C make_ascii_list
C make_iraf_list
C make_cat_list
C
      implicit none
      include 'std_io.f'
      include 'include_starnet.f'
      real*8 density,focallength,bhimaglim,blomaglim,pixelsize,
     $     rasign,rah,ram,ras,decsign,decd,decm,decs,rahalf,
     $     dechalf,ra,dec,radius,rhimaglim,rlomaglim,ihimaglim,
     $     ilomaglim
      integer inlu,lnblnk,bptr,limit,region_count,
     $     list_ra,list_dec,list_mag,list_length,batch
      logical error,round_aper,ascending,done,header
      character*64 infilename,outfilename,token,
     $     delimiters,catmap
      character*80 instr
      character*1 outstyle,sortfield
      dimension list_ra(maxlist),list_mag(maxlist),list_dec(maxlist)
      if (infilename .eq. ' ') then
         inlu=stdin
      else
         inlu=206
         open(unit=inlu,
     $        file=infilename,
     $        access='sequential',
     $        err=401,
     $        status='old')
      end if
      done=.false.
      do while (.not. done) 
         read(unit=inlu,fmt=1,end=501,err=402) instr
 1       format(a)
         delimiters=' '
         bptr=1
         limit=lnblnk(instr)+1
C
C Get the RA aperture center (hours)
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,rah,error)
         if (error) goto 400
         if (rah .lt. 0.0d0) then
            rasign=-1.0d0
            rah=-rah
         else
            rasign=1.0d0
         end if
C
C Get the RA aperture center (minutes)
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,ram,error)
         if (error) goto 400
         if (ram .lt. 0.0d0) then
            error=.true.
            call err_msg('bmain',
     $           'A sign is not allowed on RA minutes')
            goto 400
         end if
C
C Get the RA aperture center (seconds)
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,ras,error)
         if (error) goto 400
         if (ras .lt. 0.0d0) then
            error=.true.
            call err_msg('bmain',
     $           'A sign is not allowed on RA seconds')
            goto 400
         end if
C
C Get the Decl aperture center (deg)
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,decd,error)
         if (error) goto 400
         if (decd .lt. 0.0d0) then
            decsign=-1.0d0
            decd=-decd
         else
            decsign=1.0d0
         end if
C
C Get the Decl aperture center (minutes)
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,decm,error)
         if (error) goto 400
         if (decm .lt. 0.0d0) then
            error=.true.
            call err_msg('bmain',
     $           'A sign is not allowed on Decl minutes')
            goto 400
         end if
C
C Get the Decl aperture center (seconds)
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,decs,error)
         if (error) goto 400
         if (decs .lt. 0.0d0) then
            error=.true.
            call err_msg('bmain',
     $           'A sign is not allowed on Decl seconds')
            goto 400
         end if
C
C Get the RA half-width
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,rahalf,error)
         if (error) goto 400
         rahalf=pio180*rahalf/3600.0d0
         if ((rahalf .lt. 0.0d0) .or. (rahalf .gt. pi/4.0d0)) then
            error=.true.
            call err_msg('bmain',
     $           'RA half-width must lie between 0 and 90 deg.')
            goto 400
         end if
C
C Get the Decl half-width
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,dechalf,error)
         if (error) goto 400
         dechalf=pio180*dechalf/3600.0d0
         if ((dechalf .lt. 0.0d0) .or. (dechalf .gt. pi/4.0d0)) then
            error=.true.
            call err_msg('bmain',
     $           'Decl. half-width must lie between 0 and 90 deg.')
            goto 400
         end if
C
C Get the red low magnitude
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,rlomaglim,error)
         if (error) goto 400
         if ((rlomaglim .lt. -5.0d0) .or.
     $        (rlomaglim .gt. 30.0d0)) then
            error=.true.
            call err_msg('bmain',
     $           'The red low magnitude limit must be '//
     $           'between -5 and 30.')
            goto 400
         endif
C
C Get the red high magnitude
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,rhimaglim,error)
         if (error) goto 400
         if ((rhimaglim .lt. -5.0d0) .or.
     $        (rhimaglim .gt. 30.0d0)) then
            error=.true.
            call err_msg('bmain',
     $           'The red high magnitude limit must be '//
     $           'between -5 and 30.')
            goto 400
         endif
         if (rhimaglim .le. rlomaglim) then
            error=.true.
            call err_msg('bmain',
     $           'The red low mag limit must be '//
     $           'less than the high mag limit.')
            goto 400
         end if
C
C Get the blue low magnitude
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,blomaglim,error)
         if (error) goto 400
         if ((blomaglim .lt. -5.0d0) .or.
     $        (blomaglim .gt. 30.0d0)) then
            error=.true.
            call err_msg('bmain',
     $           'The blue low magnitude limit must be '//
     $           'between -5 and 30.')
            goto 400
         endif
C
C Get the blue high magnitude
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,bhimaglim,error)
         if (error) goto 400
         if ((bhimaglim .lt. -5.0d0) .or.
     $        (bhimaglim .gt. 30.0d0)) then
            error=.true.
            call err_msg('bmain',
     $           'The blue high magnitude limit must be '//
     $           'between -5 and 30.')
            goto 400
         endif
         if (bhimaglim .le. blomaglim) then
            error=.true.
            call err_msg('bmain',
     $           'The blue low mag limit must be '//
     $           'less than the high mag limit.')
            goto 400
         end if
C
C Get the index low magnitude
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,ilomaglim,error)
         if (error) goto 400
         if ((ilomaglim .lt. -5.0d0) .or.
     $        (ilomaglim .gt. 30.0d0)) then
            error=.true.
            call err_msg('bmain',
     $           'The index low magnitude limit must be '//
     $           'between -5 and 30.')
            goto 400
         endif
C
C Get the index high magnitude
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,ihimaglim,error)
         if (error) goto 400
         if ((ihimaglim .lt. -5.0d0) .or.
     $        (ihimaglim .gt. 30.0d0)) then
            error=.true.
            call err_msg('bmain',
     $           'The index high magnitude limit must be '//
     $           'between -5 and 30.')
            goto 400
         endif
         if (ihimaglim .le. ilomaglim) then
            error=.true.
            call err_msg('bmain',
     $           'The index low mag limit must be '//
     $           'less than the high mag limit.')
            goto 400
         end if
C
C Get the density
C
         call get_next_token(instr,bptr,limit,token,delimiters)
         call str2r8(token,density,error)
         if (error) goto 400
         if (density .lt. 0.0d0) then
            error=.true.
            call err_msg('bmain',
     $           'The density must be greater than zero.')
            goto 400
         endif
C
C Assemble the RA and Decl aperture center values
C
         ra=rasign*(rah+ram/60.0d0+ras/3600.0d0)*15.0d0*pio180
         if ((ra .lt. 0.0d0) .or. (ra .gt. twopi)) then
            error=.true.
            call err_msg('bmain','RA must lie between 0 and 24 hr.')
         end if
         dec=decsign*(decd+decm/60.0d0+decs/3600.0d0)*pio180+pio2
         if ((dec .lt. 0.0d0) .or. (dec .gt. pi)) then
            error=.true.
            call err_msg('bmain',
     $           'Decl. must lie between -90 and 90 deg.')
         end if
C
C Now go find the stars.
C
         if (.not. error) then
            region_count=region_count+1
            call find_stars(ra,dec,rahalf,dechalf,
     $           rlomaglim,rhimaglim,blomaglim,bhimaglim,
     $           ilomaglim,ihimaglim,density,
     $           list_ra,list_dec,list_mag,list_length,catmap,error)
            if (error) goto 400
            if (outstyle .eq. 'a') then
               call make_ascii_list(list_ra,list_dec,list_mag,
     $              list_length,dec,ra,round_aper,focallength,
     $              pixelsize,ascending,sortfield,outfilename,
     $              radius,region_count,rahalf,dechalf,error)
               if (error) goto 400
            elseif (outstyle .eq. 'i') then
               call make_iraf_list(list_ra,list_dec,list_mag,
     $              list_length,dec,ra,round_aper,focallength,
     $              pixelsize,ascending,sortfield,outfilename,
     $              radius,region_count,rahalf,dechalf,error)
               if (error) goto 400
            elseif (outstyle .eq. 'c') then
               call make_cat_list(list_ra,list_dec,list_mag,
     $              list_length,dec,ra,round_aper,focallength,
     $              pixelsize,ascending,sortfield,outfilename,
     $              radius,region_count,rahalf,dechalf,error)
               if (error) goto 400
            end if
         end if
      end do
 501  if (infilename .ne. ' ') then
         close(unit=inlu,err=403)
      end if
      goto 400
 401  error=.true.
      call err_msg('imain','Error opening region specification file.')
      goto 400
 402  error=.true.
      call err_msg('imain','Error reading region specification.')
      goto 400
 403  error=.true.
      call err_msg('imain','Error closing region specification file.')
 400  continue
      if (error) then
         call err_msg('bmain','Traceback')
      end if
      return
      end
